/*
   Binary   Decimal    Hex   ASCII
   01010101   85       55      U

\t - tab (HT/TAB), same as \x09
\n - newline (NL), same as \x0a
\r - car.return (CR), same as \x0d
\f - form feed (FF), same as \x0c
\a - alarm (bell) (BEL), same as \x07
\e - escape (ESC) , same as \x1b

Table des 128 caractres ASCII
Dans cette table, les 33 caractres de contrle (codes 0  31 et 127)
sont prsents avec leur nom en anglais suivi d'une traduction entre parenthses.
Code
                   en    base    | Caractre  | Signification
10 | 8     |  16    |   2        |
0    0        00        0000000      NUL         Null (nul)
1    01       01        0000001      SOH    Start of Header (dbut d'en-tte)
2    02       02        0000010      STX    Start of Text (dbut du texte)
3    03       03        0000011      ETX    End of Text (fin du texte)
4    04       04        0000100      EOT    End of Transmission (fin de transmission)
5    05       05        0000101      ENQ    Enquiry (demande)
6    06       06        0000110      ACK    Acknowledge (accus de rception)
7    07       07        0000111      BEL    Bell (caractre d'appel)
8    010      08        0001000      BS     Backspace (espacement arrire)
9    011      09        0001001      HT     Horizontal Tab (tabulation horizontale)
10   012      0A        0001010      LF     Line Feed (saut de ligne)
11   013      0B        0001011      VT     Vertical Tab (tabulation verticale)
12   014      0C        0001100      FF     Form Feed (saut de page)
13   015      0D        0001101      CR     Carriage Return (retour chariot)
14   016      0E        0001110      SO     Shift Out (fin d'extension)
15   017      0F        0001111      SI     Shift In (dmarrage d'extension)
16   020      10        0010000      DLE    Data Link Escape
17   021      11        0010001      DC1    Device Control 1  4
18   022      12        0010010      DC2    (DC1 et DC3 sont gnralement utiliss pour coder
19   023      13        0010011      DC3     XON et XOFF dans un canal de communication duplex)
20   024      14        0010100      DC4
21   025      15        0010101      NAK    Negative Acknowledge (accus de rception ngatif)
22   026      16        0010110      SYN    Synchronous Idle
23   027      17        0010111      ETB    End of Transmission Block (fin du bloc de transmission)
24   030      18        0011000      CAN    Cancel (annulation)
25   031      19        0011001      EM     End of Medium (fin de support)
26   032      1A        0011010      SUB    Substitute (substitution)
27   033      1B        0011011      ESC    Escape (chappement)
28   034      1C        0011100      FS     File Separator (sparateur de fichier)
29   035      1D        0011101      GS     Group Separator (sparateur de groupe)
30   036      1E        0011110      RS     Record Separator (sparateur d'enregistrement)
31   037      1F        0011111      US     Unit Separator (sparateur d'unit)
*/

#define Manchester_NUL           0x00  // NUL   : Null (nul)
#define Manchester_SOH           0x01  // SOH   : Start of Header (dbut d'en-tte)
#define Manchester_STX           0x02  // STX   : Start of Text (dbut du texte)
#define Manchester_ETX           0x03  // ETX   : End of Text (fin du texte)
#define Manchester_EOT           0x04  // EOT   : End of Transmission (fin de transmission)
#define Manchester_ESC           0x1b  // ESC    Escape (chappement) --> fonction
#define Manchester_Synchro       0x55  // ASCII : U
// Utilisation des caratres Device Control 1  4
#define Manchester_DC1           0x11  // 1er paramettre
#define Manchester_DC2           0x12  // 2ieme paramettre
#define Manchester_DC3           0x13  // 3ieme paramettre
#define Manchester_DC4           0x14  // 4ieme paramettre

#define Manchester_CMD           0x20

#define NB_LINE_LCD       2
#define NB_COLONNE_LCD   16

void Send_Manchester_TXT(char row, char column, char *s1)
{
 char index, character;

 /************* Manchester *************/
    Man_Send(Manchester_Synchro);  // Send "Synchro" byte
    Man_Send(Manchester_Synchro);  // Send "Synchro" byte
    Delay_ms(10);                  // Wait for a while
    Man_Send(Manchester_SOH);      // Send "start" byte
    Delay_ms(10);                  // Wait for a while
    
    Man_Send(48+row);      // Send "row" byte
    Delay_ms(10);                  // Wait for a while
    Man_Send(48+column);   // Send "column" byte
    Delay_ms(10);                  // Wait for a while
    //
    index = 0;                     // Initialize index variable
    character = s1[index];       // Take next char from string
    
    while(character)
    {            // String ends with zero
      Man_Send(character);         // Send character
      Delay_ms(10);                // Wait for a while
      index++;                     // Increment index variable
      character = s1[index];       // Take next char from string
    }
    // fin Trame de Donnes
    Man_Send(Manchester_EOT);                // Send "end" byte
    Delay_ms(10);
   /*********** Manchester END ***********/
}

void Send_Manchester_CMD(char CMD)
{
 char s1[2];
 
 s1[0] = Manchester_CMD + CMD;
 s1[1] = '\0';
 Send_Manchester_TXT(0,0,s1);
}

int attente_balise(unsigned int balise)
{
 int err, temp;
 while(1)
   {                             // Wait for the "start" byte
    temp = Man_Receive(&err);  // Attempt byte receive
    if(temp == balise)
       break;      // "balise" byte, see Transmitter
                 // We got the starting sequence
    if(err) break;   // Exit so we do not loop forever
   }
 return err;
}

int Manchester_receive()
{
  int error = 0, ErrorCount = 0;
  char temp;
  unsigned char ligne_ = 1;
  unsigned char colonne_ = 1;

  while(1)
  {  // Endless loop
   // ******** Dbut Zone SOH -- Dbut Trame de donnes ***********
   error = attente_balise(Manchester_SOH);
   if(error)
     {   // If error occurred
       ErrorCount++;              // Update error counter
     }
   //------ ROW ------------
   temp = Man_Receive(&error);  // Attempt byte receive

     if(error)
     {   // If error occurred
       ErrorCount++;              // Update error counter
     }
     
   ligne_ = temp - 48;
   //-------column-----------
   temp = Man_Receive(&error);  // Attempt byte receive

     if(error)
     {   // If error occurred
       ErrorCount++;              // Update error counter
     }
     
   colonne_ = temp - 48;
   //------------------------
   do
   {
     temp = Man_Receive(&error);  // Attempt byte receive

     if(error)
     {   // If error occurred
       ErrorCount++;              // Update error counter
     }
     else
     {// No error occurred
      // Lcd Command Purpose
      if((ligne_ == 0)&&(colonne_ == 0))
      {
       Lcd_Cmd(temp - Manchester_CMD);
      }
      else
      {
       switch(temp)
       {
        case Manchester_EOT:
        break;
       
        default:
         Lcd_Chr(ligne_, colonne_, temp);
         colonne_++;
         if(colonne_ == NB_COLONNE_LCD) colonne_ = 1, ligne_++;
         if(ligne_ == NB_LINE_LCD+1) ligne_ = 1;
       }
      }
     }
    }
    while(temp != Manchester_EOT); // If "Manchester_EOT - Fin TRAME des Donnes" byte was received exit do loop - fin text

    // ************ Fin TRAME des Donnes **************

    if(ErrorCount > 0)//20)
    {                          // In case of multiple errors
     //Lcd_Chr_CP('?');
     //Lcd_Chr_CP(ErrorCount+48);
     Man_Synchro();    // Try to synchronize again
        //Man_Receive_Init();    // Alternative, try to Initialize Receiver again
        ErrorCount = 0;          // Reset error counter
    }
   }
}
/*
Library LCD Routines :
Lcd_Init
void Lcd_Out(char row, char column, char *text);
void Lcd_Out_Cp(char *text);
void Lcd_Chr(char row, char column, char out_char);
void Lcd_Chr_Cp(char out_char);
void Lcd_Cmd(char out_char);
*/
// LCD Manchester Routines :
#define Manchester_Lcd_Out(row, column, text)      Send_Manchester_TXT(row, column, text)
#define Manchester_Lcd_Chr(row, column, out_char)  Send_Manchester_TXT(row, column, out_char)
#define Manchester_Lcd_Cmd(out_char)               Send_Manchester_CMD(out_char)